import java.awt.*;      
import java.net.*;   

/**
 * Board : 
 * jGames Boards' super class
 * <PRE>
 * this contain all the needed methods to build a Board for jGames 
 *
 * </PRE>
 * @version 1.0b1 25/04/98
 * @author Pierre-Mikael Legris (plegris@fbwww.epfl.ch)
 */
class Board extends Canvas {
  public int tab[];
  public int myW ;
  public int myH ;
  public boolean horizBoard ;
  int tH ;
  int tW ;
  Dimension mysize = new Dimension(myW,myH);
  boolean initDone = false;
  Gpanel owner;
 
  /**
   * constructror
   */
  public Board () {   
  }

  /**
   * this is called by game boards to be sure all the vars are 
   * set right in the super class
   */
  public void setVars(Gpanel o,int t[],int W, int H,boolean h) {
    this.owner = o;
    this.tab = t;
    this.myW = W;
    this.myH = H;    
    this.horizBoard = h;
  }

  /**
   * This overwrite the Component update(Graphics g) method that 
   * normally clean up the Canvas making it flicker
   */
  public void update(Graphics g) {
    paint(g);
  }

  /**
   * paint() create a new Graphics on which we will draw offscreen
   * then it displays this graphics when it's finished. This avoid
   * flickering and is called double buffering
   */
  public void paint(Graphics g) {
    Image preI = createImage(myW,myH);
    Graphics pre = preI.getGraphics();
    paintBoard(pre);
    g.drawImage(preI,tW(),tH(), this); 
  }

  /**
   * This method must be overwriten by the Game Board, this where you
   * draw the board
   */
  public void paintBoard(Graphics pre){ }
  
  /**
   * This method must be overwriten by the Game Board, from there you gives
   * vars to the super class. and do all that needed at start such as loading
   * images
   */
  public void init (Gpanel owner) {
  }

  /**
   * This method is in charge of downloading images for you.
   * Login.sURL is the URL of the directory from where this 
   * applet was downloaded.
   */
  public Image getImage(String imageName) {
    URL tempURL = null;
    Image tempImage = null;
    try {
      tempURL = new URL(Login.sURL+imageName);
      tempImage =  getToolkit().getImage(tempURL);   
    } catch( MalformedURLException e ) {
      System.out.println("image URL is bad "+ e); 
    }
    System.out.println("downloaded "+tempURL);
    return tempImage;
  }

  /**
   * This method calcul the position where to draw the Board (left corner)
   */
  int tW() {
    int temp = (this.size().width - myW) / 2;
    if (temp < 0 ) { temp = 0 ;}
    return temp; 
  }

  /**
   * This method calcul the position where to draw the Board (top corner)
   */
  int tH() {
    int temp = (this.size().height- myH) / 2;
    if (temp < 0 ) { temp = 0 ;}
    return temp;
  }
  
  /**
   * catch mouse clicks and send x,y coordinates of the click in the
   * Image defined by the user, to getClickValue.
   */
  public boolean mouseDown(Event evt, int xC, int yC) { //DNC
    getClickValue(xC - tW(),yC - tH()); 
    return true; 
  }
  
  /**
   * this method should be overwriten, this is from where you give
   * the value of the click to Game Panel
   */
  public void getClickValue(int xC,int yC) { }


  /**
   * This is just needed
   */
  public Dimension minimumSize() {
    return mysize;
  }
  
  /**
   * This is just needed
   */
  public Dimension preferredSize() {
    return minimumSize();
  }

  
}






