
import java.awt.*;
import java.net.*;
import java.util.StringTokenizer;
import java.util.Hashtable;

/**
 *@see Login 
 *@author Pierre-Mikael Legris (plegris@fbwww.epfl.ch)
 */ 
public class Gpanel extends Frame implements Runnable {
  // UI vars
  TextArea status ;
  Label header = new Label("Playing room",Label.CENTER);
  Label move = new Label("MOVE");
  TextField chat = new TextField();
  Button stop = new Button("Stop");
  Font defFont = new Font("Helvetica", Font.PLAIN , 12);
  FontMetrics dFM = getFontMetrics(defFont);
  Board gameBoard ;
  int tab[] ; 
  boolean dead = false;
  boolean horizBoard = true;
  boolean turn = true;
  Dimension myDim = new Dimension(630,480);   

  // vars
  private Thread g; 
  static Wplayer player = null;
  String msg = "";

 

  // constructor
  public Gpanel(Wpanel owner,Wplayer human,String message,String game) {
    owner.dispose();
    this.player = human;
    this.msg = message;
    
    try {
      gameBoard = (Board) Class.forName(game).newInstance();
    } catch(Exception e) {
      System.out.println(e);
      Ebox tell = new Ebox(this,"* Can't load this game *",true);
      tell.show(true);
      die();
    }
    gameBoard.init(this);
    
    if (horizBoard) {
      status = new TextArea(9 , 100);
    } else {
      status = new TextArea(100 , 30);
    }
    
    
    Thread g = new Thread(this);
    g.start(); 
  }

  //run
  public void run() {
    
    System.out.println("Gpanel started " + player.getNick());
    makeUI();
    print("<start>"+msg);
    String r;
    while(player.isAlive() && player.playing()) {
      try {
	r = player.read();
	if (r.compareTo("") != 0) { ParseS(r) ; }
      } catch(Exception e){ }
      try{ Thread.sleep(100); } catch(InterruptedException e){ } 
    }
    System.out.println("Gpanel died");
    if (! player.isAlive()) {
      Ebox tell = new Ebox(this,"* disconected *",true);
      tell.show(true);
      die();
    }
  }
  
  // Parser
  public void ParseS(String text) {
    StringTokenizer st = new StringTokenizer(text,"<");
    if (st.countTokens() == 2) {
      char commande = st.nextToken().charAt(0);
      switch(commande) {
      case 'C': //chat
	print("<"+player.getPlay()+"> "+st.nextToken());
	break;
	
      case 'P': //part
      case 'Q': //quit
	Ebox error = new Ebox(this,""+player.getPlay()+
			      " left the game : "+st.nextToken(),true);
	error.show(true);
	die();
	break;

      case 'T': //turn string
	if (st.nextToken().compareTo("true") == 0) {
	  move.setText("PLAY");
	  move.setBackground(Color.green);
	  turn = true;
	  player.setTurn(true);
	} else {
	  move.setText("WAIT");
	  move.setBackground(Color.red);
	  player.setTurn(false);
	  turn = false;
	}
	break;

      case 'G': //game string
	parseG(st.nextToken());
	break;
	
      case 'M': //error while moving
	Ebox error2 = new Ebox(this,st.nextToken(),true);
	error2.show(true);   
	break;

      case 'W': //did you win?
	String winner = "Tie: no winner";
	String wTemp = st.nextToken();
	if (wTemp.compareTo("Y") == 0) {
	  winner = "You Win";
	} 
	if (wTemp.compareTo("N") == 0) {
	  winner = "You Lose";
	}
	Wbox win = new Wbox(this,winner,true);
	player.setTurn(false);
	win.show(true);
	break;
	
      case 'S': //server message
	print(st.nextToken());
	break;
      }
    }
  }

  //Parse the Game String and display the result
  public void parseG(String g) {
    StringTokenizer st = new StringTokenizer(g,"#");
    int i = 0;
    while (st.hasMoreTokens()) {
      tab[i] = Integer.parseInt(st.nextToken());
      i++;
    }
    gameBoard.repaint();
  }

  //play : player clicked on a bol
  public void play(int bol) {
    if (turn) {
      player.setTurn(false);
      player.send("M<"+bol);
    } else {
      Ebox error2 = new Ebox(this,"Not your turn",true);
      error2.show(true); 
    }
  }

  // Status writer
  public void print(String p)
  {
    // word warping
    int l = status.size().width - 40;
    StringTokenizer st = new StringTokenizer(p);
    String tmp = "";
    String tToken = "";
    while (st.hasMoreTokens()) {
      tToken = st.nextToken();
      if (dFM.stringWidth(tToken) > l ) {
	tToken = tmp + " " +tToken;
	while (dFM.stringWidth(tToken) > l ) {
	  tmp = "";
	  int i;
	  for (i = 0; dFM.stringWidth(tmp) < ( l - 10) ; i++ )
	    tmp = tmp + tToken.substring(i  , i + 1);
	  tToken = tToken.substring(i , tToken.length());
	  status.appendText(tmp + "\n");
	}
	status.appendText(tmp + "\n");
	tmp = tToken;
      } else {
	if (dFM.stringWidth(tmp + " " + tToken) > l ) {
	  status.appendText(tmp + "\n" );
	  tmp = tToken;
	} else {
	  tmp = tmp + " " + tToken;
	}
      }
    }
    status.appendText(tmp + "\n");
	    
    // I use this to scroll down, and cut off
    int i = status.getText().length();
    if (i > 10000) {
      status.setText(status.getText().substring( i - 7000 , i));
      i = status.getText().length();
    }
    status.select(i,i);
    repaint();
  }// end print

  
  
  // make UI
  public void makeUI() {
    status.setEditable( false );
    setLayout(new BorderLayout());
    setBackground(Color.white);
    header.setFont(new Font("Helvetica", Font.BOLD , 13));
    status.setFont(defFont);
    header.setBackground(Color.blue);
    header.setForeground(Color.white);
    add("North",header);

    Panel middle = new Panel();
     middle.setLayout(new BorderLayout());
     middle.add("Center",gameBoard);
     if (horizBoard) {
       middle.add("South",status);
     } else {
       middle.add("East",status);
     }

    add("Center",middle);

    Panel bottom = new Panel();
     bottom.setLayout(new BorderLayout());
     bottom.add("Center",chat);
     bottom.add("East",stop);
     bottom.add("West",move);

    add("South",bottom);

    header.setText(" playing room    : "+player.getNick() +
		   " vs "+ player.getPlay());

    resize(myDim.width,myDim.height); 
    repaint();
    show(true);
  }

  // leave the game
  public void part(String text) {
    player.send("P<"+text);
    die();
  }

  // die
  public void die() {
    if (! dead) {
      dead = true;
      player.send("forced");
      player.clean();
      Wpanel wait = new Wpanel(player);
      dispose();
    }
  }
  
  // handleEvent
  public boolean handleEvent(Event evt) {
    // close the windows
    if (evt.id == Event.WINDOW_DESTROY) {
      Sbox really = new Sbox(this,"Stop this game",true);
      really.show(true);   
    }
    return super.handleEvent(evt);
  }

  // handle actions
  public boolean action(Event e, Object o)
  {
    // chat
    if (e.target == chat) {
      String inText = chat.getText();
      if (inText.compareTo("") != 0) {
	chat.setText("");
	inText = inText.replace('<','\\').replace('>','/');
	player.send("C<"+inText);
	print("> "+inText);
      }
      return true;
    }
    
    // stop
    if (e.target == stop) {
      Sbox really = new Sbox(this,"Stop this game",true);
      really.show(true);   
      return true;
    }
    return true;
  }
}

















