
import java.net.*;
import java.io.*;
import java.util.StringTokenizer;

/**
 *@see ServM
 *@author Pierre-Mikael Legris (plegris@fbwww.epfl.ch)
 *
 *
 */
public class Game implements Runnable
{
  private Thread g;
  private Player player1 = null;
  private Player player2 = null;
  private String nick1 = "";
  private String nick2 = "";
  private boolean gaming = true;
  private boolean waiting = true;
  private int waitcount = 0;
  private boolean turn1 = false;
  private Rules rules = null;
  
  /** 
   * Construct a new game and launch the Thread
   */
  public Game(String n1,String n2,String r) {
    nick1 = n1;
    nick2 = n2;
    
    player1 = (Player) ServM.users.get(nick1);
    player2 = (Player) ServM.users.get(nick2);
    
    try {
      rules = (Rules) Class.forName(r).newInstance();  
      rules.setName1(n1);
    } catch(Exception e) {
      System.out.println(e);
      gaming = false;
      player1.send("S<** Can't find this game **");
      player2.send("S<** Can't find this game **");
      die();
    } 
    
    Thread g = new Thread(this);
    g.start();
  } 

  /**
   * create a newGame
   */
  private void newGame()
  {
    rules.newGame();
    gaming = true;
    waiting = false;
    waitcount = 0;
    player1.setTurn(turn1);
    player2.setTurn(! turn1);
    turn1 = ! turn1;
    player1.send("S<## new game ## score: "+player1.getScore()+"/"+player2.getScore());
    player2.send("S<## new game ## score: "+player2.getScore()+"/"+player1.getScore());
    player1.send("G<"+rules.getTabS(player1.getNick()));    
    player2.send("G<"+rules.getTabS(player2.getNick()));
  }

  //RUN
  /**
   * Reads the stream, parses the Input 
   */
  public void run()
  {
     newGame(); 
    String r1 = "";
    String r2 = "";
    while((player1.isAlive()) && (player2.isAlive()) && (gaming || waiting)) {
      try {
	if (player1.bot && (! player2.hisTurn()) && gaming) {
	  r1 = "M<"+player1.play(rules.getTab(player1.getNick()));
	} else {
	  r1 = player1.read();
	}
	if (player2.bot && (! player1.hisTurn()) && gaming) {
	  r2 = "M<"+player2.play(rules.getTab(player2.getNick()));
	} else {
	  r2 = player2.read();
	}      

       if (r1.compareTo("") != 0) { parseS(r1,player1,player2); }
       if (r2.compareTo("") != 0) { parseS(r2,player2,player1); }
       
       } catch(Exception e){ }
      try{ Thread.sleep(100); } catch(InterruptedException e){ }
      
    }
    die();
  }// run
  
  /**
   * die
   */
  void die() {
    if( player1.isAlive()) {
      if(! player2.isAlive() ) {
	player1.send("Q<Connection lost");
      }
      player1.part();
      Mstream manca1 = new Mstream( nick1 );
    }
    if( player2.isAlive()) {
      if(! player1.isAlive() ) {
	player2.send("Q<Connection lost");
      }
      player2.part();
      Mstream manca2 = new Mstream( nick2 );
    }
    log("end");
  }

  // ParseS
  /**
   * Parse the incoming String an do what's needed
   */
  void parseS(String text,Player player,Player opponent) {
    StringTokenizer st = new StringTokenizer(text,"<");
    if (st.countTokens() == 2) {
      char commande = st.nextToken().charAt(0);
      switch(commande) {
      case 'C':
	opponent.send("C<" + st.nextToken());
	break;

      case 'Q':
	player.die();
	opponent.send("Q<" + st.nextToken());
	break;
	
      case 'P':
	opponent.send("P<" + st.nextToken());
	gaming = false;
	waiting = false;
	break;

      case 'G': //ok to start a new game
	if (waiting) {
	  waitcount++;
	  if ((waitcount > 1) || opponent.bot) {
	    newGame();
	  }
	}
	break;

      case 'M': // rules managing
	if (player.hisTurn() && gaming) {
	  int bol = Integer.parseInt(st.nextToken());
	  if (! rules.verif(bol,player.getNick())) { //bad move
	    player.setTurn(true);
 	    player.send("M<Move not allowed");
	  } else {
	    // send the new table to the opponent and the player
	    boolean pPlay = true;
	    if (! rules.replay(bol,player.getNick())) { //user loses his turn
	      player.setTurn(false);
	      opponent.setTurn(true);
	      pPlay = false;
	    } else { //user replay
	      player.setTurn(true);
	      opponent.setTurn(false);
	    }
	    int ttab[] = rules.move(bol,player.getNick());
    
	    opponent.send("G<"+rules.getTabS(opponent.getNick()));
	    opponent.send("S<* "+rules.msg(bol,false));
	    player.send("G<"+rules.getTabS(player.getNick()));
	    player.send("S<* "+rules.msg(bol,true));
	    int winCode = rules.win(player.getNick()); 
	    
	    if (winCode != 0 ) {  // game finished 
	      player.setTurn(false);
	      opponent.setTurn(false);
	      gaming = false;
	      waiting = true;
	      if (winCode == 1) { //game finished
		player.setScore(player.getScore() + 1);
		player.send("W<Y");
		opponent.send("W<N");
	      }
	      if (winCode == 2) { //game finished
		opponent.setScore(opponent.getScore() + 1);
		player.send("W<N");
		opponent.send("W<Y");
	      }
	      if (winCode == -1 ) {//Tie
		player.send("W<T");
		opponent.send("W<T");
	      }
	    }
	  }
	} else {
	  player.send("M<Not your turn");
	}
	break;
      }
    }
  }
  
  // LOG
  void log(String msg)
  {
    // System.out.println("Game: "+nick1+" - "+nick2+" "+msg);
  }      
}







