import java.net.*;
import java.io.*;
import java.util.StringTokenizer;
import java.util.Hashtable;
import java.util.Enumeration;


/**
 * This is the server class for the game jMancala <BR>
 * Start it with the command :  java ServM <port><BR>
 * <BR>
 * version 1.0b1 26/04/98<BR>
 * author Pierre-Mikael Legris (plegris@fbwww.epfl.ch)<BR> 
 * 
 *
 * <B>How does it works? </B>
 * <PRE>
 *   ServM
 *     -listen to the determinate port. 
 *     -handle the user list: (nick , Human(nick,Socket,comment))
 *     -is used to broadcast messages to users.
 *    when a connection occure ServM pass the socket to SocketHub.
 *- 
 *   SocketHub(Socket)
 *    get the stream from the socket, verify the nick of the
 *    incoming user. When evrything is OK it add the user to
 *    ServM's user list with Human(nick,Socket,comment)
 *    and create a new Mstream(nick)
 *- 
 *   Human(nick,Socket,comment)
 *    read and write in the Socket
 *    have some usefull method such as isAlive()
 *-
 *   Mstream(nick)
 *    Mstream is the "waiting room" on the server for the users.
 *    It takes the control of this user, then handle the data
 *    streams between users until he starts a game.
 *    with Game(opponent,nick)
 *-
 *   Game(opponent,nick)
 *    Take the control of the 2 players: Player(nick,Human,move)
 *    and manage the game using MRules(nick1). When the game end 
 *    it gives the two users back to Mstream.
 *-
 *   Player(nick,Human,move)
 *    is an extension of Human, with some more method for Game()
 *    it handle if it's his Turn to play ...
 *-
 *   MRules(nick) (note that MRules is also used by the client)
 *    It handle the game board and the rules.
 *-       
 * </PRE>
 * This is opensource software.<BR>
 * (just notify me if you use a part of this code or modify it)
 *
 * @version 1.0b1 26/04/98
 * @author Pierre-Mikael Legris (plegris@fbwww.epfl.ch) 
 */

public class ServM implements Runnable
{
  /** 
   * users handle the uselist by binding a nick name to a Human()
   */
  static Hashtable users = new Hashtable();
  static Hashtable games = new Hashtable();
  static Hashtable bots = new Hashtable();
  int mainPort;
  ServerSocket MSocket;
  Thread server;
  
  /** 
   * Construct a new Server and launch the Thread
   */
  public ServM(int port)
  {
    mainPort = port;
    
    try {
      MSocket = new ServerSocket(mainPort);
    } catch(Exception e) {
      log("*error* " + e + "\n cannot create server socket");
      System.exit(1);
    }
    log("ServM started listening on port"+ mainPort );
    
    server = new Thread(this);
    server.setPriority(Thread.MIN_PRIORITY);
    server.start();
  } // ServM ( port )
  

  /**
   * usage: ServM <port>
   */
  public static void main(String args[])
  {
    int mainPort = 0;
    //get the port to bind
    if(args.length != 1)
      {
	log("usage: java ServM <port>");
	System.exit(1);
      } else {
	try {
	  mainPort = Integer.parseInt(args[0]);
	} catch(Exception e) {
	  log("*error*" + e);
	  log("usage: java ServM <port>");
	  System.exit(1);
	}
      }
    
    ServM me = new ServM(mainPort);
  } // Main

  // RUN
  /** 
   * wait for incoming connections and open new streams
   */
  public void run()
  {
    rehash();
    Socket mainSocket = null;
    while(true)
    {  
      try {
	mainSocket = MSocket.accept();
      } catch(Exception e) {
	log("*error*" + e + "\n accept connection failed");
	break;
      }     
      //log("connection from "+ mainSocket.getInetAddress().getHostName());
      
      // send the stuff to the hub
      SocketHub NSH = new SocketHub( mainSocket );
    }
  } // Run

  // bdS
  /**
   * Broadcast message to every clients
   */
  static void bdS(String msg)
  {
    for (Enumeration en = users.keys() ; en.hasMoreElements() ;) {
      String key = (String) en.nextElement();
      try{
      Player tempH = (Player) ServM.users.get(key);
      if (tempH.isAlive())
	tempH.send("S<"+msg);
      } catch(Exception e) { }
    }
  }


  // bdM
  /**
   * Broadcast message to persons that aren't playing
   */
  static void bdM(String msg)
  {
    for (Enumeration en = users.keys() ; en.hasMoreElements() ;) {
      String key = (String) en.nextElement();
      try{
      Player tempH = (Player) ServM.users.get(key);
      if ((! tempH.playing()) && tempH.isAlive())
	tempH.send(msg);
      } catch(Exception e) { }
    }
  }
  
  // bdU
  /**
   * Brodcast an update message
   */
  static void bdU(String key)
  {
    Player tempH = (Player) ServM.users.get(key); 
    bdM("U<"+key+">"+tempH.getInfo()+">"+tempH.getAddy()+">"+tempH.getPlay());
  }

  //rehash
  /**
   * reload jSave.dat
   */
  static void rehash() {
    boolean ok = true;
    jSave data = null;
    FileInputStream st1 = null;
    ObjectInputStream st2 = null;
    try {
      st1 = new FileInputStream("jGame.dat");
      st2 = new ObjectInputStream(st1);
      Object obj = st2.readObject();
      data = (jSave)obj;
    }
    catch(IOException ex) {
      ok = false;
      log("error while reading please login as admin and create a new data file");
    }
    catch(ClassNotFoundException ex) {
      log("Class not found"+ex);
      die();
    }
    finally {
      try {
	if (st2 != null)
	  st2.close();
	if (st1 !=null) 
	  st1.close();
      }
      catch (IOException ex) {
	log("error while reading (closing)"+ex);
	die();
      }
    }
    if (ok) {
      games = data.games;
      bots = data.computers;
      log("*** Rehashed : games list");
      int i = 1;
      try {
	for (Enumeration en=games.keys();en.hasMoreElements();i++) {
	  String key = (String) en.nextElement();
	  log(i+" "+key+": "+(String) games.get(key));
	}
      } catch(Exception e) {log(""+e); }
      createBots();
    } 
  }     

  /**
   * Update the bots in the userlist
   */
  static void createBots() {
    // remove old bots from the actual userlist
    try {
      int i = 0;
      for (Enumeration en=users.keys();en.hasMoreElements();i++) {
	String key = (String) en.nextElement();
	Player tempC = (Player) users.get(key);
	if (tempC.getAddy().compareTo("robot") == 0) {
	  if (! bots.containsKey(tempC.getNick())) {  
	    bdM("Q<"+tempC.getNick());
	    users.remove(tempC.getNick());
	    log("removed "+key+" from the list");
	  }
	}  
      } 
    } catch(Exception e) {log(""+e); }
   
    // fill with new bots
    try {
      int i = 0;
      for (Enumeration en=bots.keys();en.hasMoreElements();i++) {
	String key = (String) en.nextElement();
	Player tempC = null;
	boolean ok = true;
	try {
	  tempC = (Player) Class.forName(key).newInstance();  
	} catch(Exception e) { 
	  log("can't get rob "+key+" : " +e);
	  ok = false;
	}
	if (ok) {
	  if (! users.containsKey(tempC.getNick())) {
	    users.put(tempC.getNick(), tempC);
	    tempC.iPlay = (String) bots.get(key);
	    bdM("J<"+key);
	    bdU(key);
	    log("added "+key+" to the list");
	  }
	}
      }
    } catch(Exception e) {log(""+e); }
  }
  

 
  // LOG
  /**
   * write a msg to the log stream
   */
  static void log(String msg)
  {
    System.out.println("ServM: "+msg);
  }

  //DIE
  static void die() {
    System.exit(1);
  }
}



















